require 'faraday'
require 'json'

module Editframe
  class APIResource
    def self.base_url
      "https://#{Editframe::api_host}/v#{Editframe::api_version}/#{self::OBJECT_PATH}"
    end

    def self.all
      perform_get_request()['data']
    end

    def self.find(identifier)
      perform_get_request("#{identifier}")['data']
    end

    def self.perform_get_request(url = '', params: {}, options: {})
      response = Faraday.new(url: base_url, params: params, headers: retrieve_headers).get(url)
      JSON.parse(response.body)
    end

    def self.perform_post_request(url = '', body: {}, options: {}, is_form: false)
      headers = retrieve_headers
      request = Faraday.new(url: base_url, headers: headers)
      if is_form 
        headers.delete('Accept')
        headers.delete('Content-Type')
        request = Faraday.new(url: base_url, headers: headers) do |f|
          f.request :multipart
        end
      end
      request = request.post(url, body)
      JSON.parse(request.body)
    end

    def self.retrieve_headers
      headers = { 
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer #{Editframe::api_key}",
        'Editframe-Client-Id' => "#{Editframe.client_id}"
      }
      headers
    end
  end
end 
