module Editframe
  class VideoBuilder

    attr_reader :layers

    def initialize(options = {})
      @form = {}
      @layers = []
      @options = options || { background_color: 'black', resolution: '700x700', duration: 10 }
    end

    def add_audio(source, options = {})
      layer = add_layer({ :type => 'audio', **options })
      add_asset(layer[:id], source)
      layer
    end

    def add_image(source, options = {})
      layer = add_layer({ :type => 'image', **options })
      add_asset(layer[:id], source)
      layer
    end

    def add_text(text, options = {})
      layer = add_layer({ :type => 'waveform', :text => text, **options })
      layer
    end

    def add_video(source, options = {})
      layer = add_layer({ :type => 'video', **options })
      add_asset(layer[:id], source)
      layer
    end

    def add_waveform(options = {})
      layer = add_layer({ :type => 'waveform', **options })
      layer
    end
                                                            
    def encode
      @form[:config] = Faraday::ParamPart.new(generate_config, 'application/json')
      Editframe::Video.encode(@form)
    end

    private def add_asset(id, source)
      case source
      when String
        @form["url#{id}"] = source
      when File
        @form["file#{id}"] = Faraday::FilePart.new source, 'application/octet-stream'
      end
    end 

    private def add_layer(options = {})
      layer = { id: Util::uuid, **options }
      @layers.push layer 
      layer
    end 

    private def generate_config 
      options = @options
      dimensions = Util::dimensions_for_resolution options[:resolution]
      if options.key? :aspect_ratio
        hd = options.key? :hd ? options[:hd] : false
        dimensions = Util::dimensions_for_aspect_ratio(options[:aspect_ratio], hd)
      end
      options[:hd] = dimensions[:width] >= 1024
      options.delete(:resolution)
      options.stringify_keys!
      options.deep_transform_keys! { |key| key.camelize :lower }
      { **options, :dimensions => dimensions, :layers => @layers }.to_json
    end 
  end 
end
