module Editframe
  class VideoClip

    attr_reader :layers

    def initialize(source, options = {})
      @id = Util.uuid
      @form = {}
      @filters = []
      @layers = []
      @options = options || { }
      @resolution = { width: nil, height: nil }
      @trim = { start: 0, end: nil }
      @volume = 1.0
      configure_source source
    end

    def filter(name, options = {})
      @filters.push({ :name => name, :options => { **options } })
      self
    end

    def resize(resolution)
      return unless !resolution.nil?
      set_resolution resolution
      self
    end

    def set_resolution(resolution)
      return unless !resolution.nil?
      values = resolution.split('x')
      @resolution = { width: values[0].to_i, height: values[1].to_i }
      self
    end

    def set_volume(volume)
      @volume = volume.to_f
      if volume > 1 then @volume = 1.0 end
      if volume < 1 then @volume = 0.0 end
      self
    end

    def trim(trim_start, trim_end)
      @trim[:start] = trim_start
      @trim[:end] = trim_end
      if @trim[:start] < 0 
        @trim[:start] = 0
      end
      if @trim[:end] < 0 
        @trim.delete(:end)
      end
      self
    end

    def mute 
      set_volume 0 
      self
    end
                                            
    def encode
      @form[:config] = Faraday::ParamPart.new(generate_config, 'application/json')
      Editframe::Video.encode_from_clip(@form)
    end

    private def configure_source(source)
      case source
      when String
        @form["url#{@id}"] = source
      when File
        @form["file#{@id}"] = Faraday::FilePart.new source, 'application/octet-stream'
      end
      @source = source
    end 

    private def generate_config 
      clip = { :id => @id, :filters => @filters, :volume => @volume }
      options = @options
      if !@trim[:end].nil?
        clip[:trim] = @trim
      end
      if (!@resolution[:width].nil?) and (!@resolution[:height].nil?)
        clip[:resolution] = @resolution
      end
      options.stringify_keys!
      options.deep_transform_keys! { |key| key.camelize :lower }
      { **options, :layers => @layers, :clip => clip }.to_json
    end 
  end 
end
